@@ -40,8 +40,21 @@ public class Config implements Serializable, Iterable<String> {
     //     ---------------+-----------
     //                5.1 | 5.0
     //     ---------------+-----------
+    //                5.2 | 5.1, 5.0
+    //     ---------------+-----------
+    //                5.3 | 5.2, 5.1, 5.0
+    //     ---------------+-----------
+    //                5.4 | 5.3, 5.2, 5.1, 5.0
+    //     ---------------+-----------
+    public static final String[] COMPATIBLE_WITH_5_4 = {"5.3", "5.2", "5.1", "5.0"};
+    public static final String[] COMPATIBLE_WITH_5_3 = {"5.2", "5.1", "5.0"};
+    public static final String[] COMPATIBLE_WITH_5_2 = {"5.1", "5.0"};
     public static final String[] COMPATIBLE_WITH_5_1 = {"5.0"};
-    public static final Map<String, String[]> DATABASE_COMPATIBLE_VERSIONS = Map.of("5.1", COMPATIBLE_WITH_5_1);
+    public static final Map<String, String[]> DATABASE_COMPATIBLE_VERSIONS = Map.of("5.1", COMPATIBLE_WITH_5_1
+                                                                                    , "5.2", COMPATIBLE_WITH_5_2
+                                                                                    , "5.3", COMPATIBLE_WITH_5_3
+                                                                                    , "5.4", COMPATIBLE_WITH_5_4
+                                                                                );
 
     private static final long serialVersionUID = 877453207407217465L;
 
@@ -67,6 +80,7 @@ public class Config implements Serializable, Iterable<String> {
     double lofIgnoreProteinCodingBefore;
     double lofDeleteProteinCodingBases;
     String configFileName = "";
+    String configFileCanonicaPath;
     String configDirPath = ""; // Configuration file directory
     String dataDir; // Directory containing all databases and genomes
     String genomeVersion;
@@ -146,22 +160,19 @@ public class Config implements Serializable, Iterable<String> {
      * Extract and create codon tables
      */
     void createCodonTables(String genomeId, Properties properties) {
-        //---
         // Read codon tables
-        //---
         for (Object k : properties.keySet()) {
             String key = k.toString().trim();
             if (key.startsWith(KEY_CODON_PREFIX)) {
                 String name = key.substring(KEY_CODON_PREFIX.length());
                 String table = properties.getProperty(key);
+                if (debug) Log.debug("Reading codon table '" + name + "'");
                 CodonTable codonTable = new CodonTable(name, table);
                 CodonTables.getInstance().add(codonTable);
             }
         }
 
-        //---
         // Assign codon tables for different genome+chromosome
-        //---
         for (Object key : properties.keySet()) {
             String keyStr = key.toString();
             if (keyStr.endsWith(KEY_CODONTABLE_SUFIX) && keyStr.startsWith(genomeId + ".")) {
@@ -187,10 +198,13 @@ public class Config implements Serializable, Iterable<String> {
                     // Find chromosome
                     Chromosome chr = gen.getOrCreateChromosome(chromo);
 
+                    if( verbose ) Log.info("Codon table '" + codonTableName + "' assigned to chromosome '" + chr.getChromosomeName() + "'");
+
                     // Everything seems to be OK, go on
                     CodonTables.getInstance().set(genomeById.get(genomeId), chr, codonTable);
                 } else {
                     // Set genome-wide chromosome table
+                    Log.info("Codon table '" + codonTableName + "' for genome '" + genomeId + "'");
                     CodonTables.getInstance().set(genomeById.get(genomeId), codonTable);
                 }
             }
@@ -213,7 +227,7 @@ public class Config implements Serializable, Iterable<String> {
     /**
      * Build the URL for downloading a database file
      * <p>
-     * Format  : DatabaseRepository / v VERSION / snpEff_v VERSION _ genomeVersion .zip
+     * Format  : DatabaseRepository / "v{VERSION}" / "snpEff_v{VERSION}_{genomeVersion}.zip"
      * Example : http://downloads.sourceforge.net/project/snpeff/databases/v2_0_3/snpEff_v2_0_3_EF3.64.zip
      */
     public URL downloadUrl(String genomeVer, String version) {
@@ -243,6 +257,35 @@ public class Config implements Serializable, Iterable<String> {
         }
     }
 
+    /**
+     * Find config file in different locations
+     * @param configFileName
+     * @return Canonical path to config file
+     */
+    String findConfigFile(String configFile) {
+        try {
+            // Build error message
+            File confFile = new File(configFile);
+            if (verbose) Log.info("Looking for config file: '" + confFile.getCanonicalPath() + "'");
+            if( confFile.exists() && confFile.canRead() ) return confFile.getCanonicalPath();
+
+            // Absolute path? Nothing else to do...
+            if (confFile.isAbsolute())
+                throw new RuntimeException("Cannot find config file '" + confFile.getCanonicalPath() + "'");
+
+            // Try reading from current execution directory
+            String confPath = getRelativeConfigPath() + "/" + configFile;
+            confFile = new File(confPath);
+            if( confFile.exists() && confFile.canRead() ) return confFile.getCanonicalPath();
+
+            throw new RuntimeException("Cannot find config file '" + configFile + "'\n");
+        } catch (FileNotFoundException e) {
+            throw new RuntimeException("Cannot find config file '" + configFile + "'");
+        } catch (IOException e) {
+            throw new RuntimeException(e);
+        }
+    }
+
     /**
      * Genes file path (no extension)
      */
@@ -276,6 +319,10 @@ public class Config implements Serializable, Iterable<String> {
         return configFileName;
     }
 
+    public String getConfigFileCanonicaPath() {
+        return configFileCanonicaPath;
+    }
+
     public String getCoordinates() {
         String coords = getString(genomeVersion + "." + Config.KEY_COORDINATES);
         if (coords != null) return coords;
@@ -471,6 +518,29 @@ public class Config implements Serializable, Iterable<String> {
         return nameById.get(genomeVersion);
     }
 
+    public String getProperty(String property) {
+        if (!properties.containsKey(property)) {
+            if( isDebug() ) Log.warning("Property '" + property + "' not found in config file '" + getConfigFileCanonicaPath() + "'");
+            return null;
+        } 
+        return properties.getProperty(property);
+    }
+
+    /**
+	 * Parse a comma separated property as a string array
+	*/
+	public String[] propertyToStringArray(String attr) {
+		String value = properties.getProperty(attr);
+		if (value == null) return new String[0];
+
+		String values[] = value.split("[\\s+,]");
+		LinkedList<String> list = new LinkedList<>();
+		for (String val : values)
+			if (val.length() > 0) list.add(val);
+
+		return list.toArray(new String[0]);
+	}
+
     public String getReference(String genomeVersion) {
         return referenceById.get(genomeVersion);
     }
@@ -526,10 +596,11 @@ public class Config implements Serializable, Iterable<String> {
         this.genomeVersion = genomeVersion;
         this.dataDir = dataDir;
 
-        readConfig(configFileName, override); // Read config file and get a genome
+        configFileCanonicaPath = findConfigFile(configFileName);
+        readConfig(configFileCanonicaPath, override); // Read config file and get a genome
         genome = genomeById.get(genomeVersion); // Set a genome
         if (!genomeVersion.isEmpty() && (genome == null))
-            throw new RuntimeException("No such genome '" + genomeVersion + "'");
+            throw new RuntimeException("No such genome '" + genomeVersion + "' in config file '" + configFileCanonicaPath + "'");
 
         // Make this the current singleton instance
         configInstance = this;
@@ -649,18 +720,20 @@ public class Config implements Serializable, Iterable<String> {
      *
      * @return true if success
      */
-    boolean loadProperties(String configFileName) {
+    boolean loadProperties(String configPath) {
         try {
-            File confFile = new File(configFileName);
+            File confFile = new File(configPath);
             if (verbose) Log.info("Reading config file: " + confFile.getCanonicalPath());
 
-            if (Gpr.canRead(configFileName)) {
+            if (Gpr.canRead(configPath)) {
                 // Load properties
-                properties.load(new FileReader(confFile));
+                var freader = new FileReader(confFile);
+                properties.load(freader);
 
                 // Set config directory
                 configDirPath = confFile.getCanonicalFile().getParent();
 
+                freader.close();
                 return true;
             }
         } catch (Exception e) {
@@ -686,11 +759,11 @@ public class Config implements Serializable, Iterable<String> {
     /**
      * Read configuration file and create all 'genomes'
      */
-    private void readConfig(String configFileName, Map<String, String> override) {
+    private void readConfig(String configPath, Map<String, String> override) {
         //---
         // Read properties file
         //---
-        configFileName = readProperties(configFileName, override);
+        readProperties(configPath, override);
 
         //---
         // Set attributes
@@ -749,6 +822,8 @@ public class Config implements Serializable, Iterable<String> {
         // Genome specified?
         if (!genomeVersion.isEmpty()) {
             readGenomeConfig(genomeVersion, properties); // Read configuration file for genome version (if any)
+            genome = new Genome(genomeVersion, this); // Create genome object
+            genomeById.put(genomeVersion, genome);
             createCodonTables(genomeVersion, properties); // Codon tables
         }
 
@@ -780,53 +855,20 @@ public class Config implements Serializable, Iterable<String> {
             if (debug)
                 System.err.println("File '" + genomePropsFileName + "' not found"); // File does not exists? => OK
         }
-
-        genome = new Genome(genVer, properties);
-        genomeById.put(genVer, genome);
-    }
-
-    /**
-     * Reads a properties file
-     *
-     * @return The path where config file is located
-     */
-    String readProperties(String configFileName) {
-        properties = new Properties();
-        try {
-            // Build error message
-            File confFile = new File(configFileName);
-            if (loadProperties(configFileName)) return configFileName;
-
-            // Absolute path? Nothing else to do...
-            if (confFile.isAbsolute())
-                throw new RuntimeException("Cannot read config file '" + confFile.getCanonicalPath() + "'");
-
-            // Try reading from current execution directory
-            String confPath = getRelativeConfigPath() + "/" + configFileName;
-            confFile = new File(confPath);
-            if (loadProperties(confPath)) return confPath;
-
-            throw new RuntimeException("Cannot read config file '" + configFileName + "'\n");
-        } catch (FileNotFoundException e) {
-            throw new RuntimeException("Cannot find config file '" + configFileName + "'");
-        } catch (IOException e) {
-            throw new RuntimeException(e);
-        }
     }
 
     /**
      * Read config file
      */
-    String readProperties(String configFileName, Map<String, String> override) {
-        String configFile = readProperties(configFileName);
+    void readProperties(String configPath, Map<String, String> override) {
+        properties = new Properties();
+        if (!loadProperties(configPath)) throw new RuntimeException("Cannot read config file '" + configPath + "'\n");
 
         if (override != null) {
             for (String key : override.keySet()) {
                 properties.setProperty(key, override.get(key));
             }
         }
-
-        return configFile;
     }
 
     /**
